import requests
from ..constants import FORM_TEMPLATES_URL, FORMS_URL, PDF_URL, FIELDS_URL, FORM_INSTANCE_URL, RELATE_FORM_URL, \
                        RELATE_DOCUMENT_URL, RELATE_PROJECT_URL, UNRELATE_FORM_URL, UNRELATE_DOCUMENT_URL, \
                        UNRELATE_PROJECT_URL, VV_LOGIN_URL, RETURN_URL, HIDE_MENU_URL, DOCUMENTS_URL


class FormService:
    def __init__(self, vault):
        self.vault = vault

    def get_form_templates(self, query=''):
        """
        get all form templates or search by query string
        :param query: string, example: "name='TestForm'"
        :return: dict
        """
        request_url = self.vault.base_url + FORM_TEMPLATES_URL

        # check for query string parameter
        if query:
            request_url += '?q=' + query

        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_form_template(self, template_id):
        """
        get a form template by id
        :param template_id: string uuid4
        :return: dict
        """
        endpoint = FORM_TEMPLATES_URL + '/' + template_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_form_instance(self, template_id, instance_id, query_string=''):
        """
        get a form instance by form template and instance ids
        :param template_id: string uuid4
        :param instance_id: string uuid4
        :param query_string: string, example: 'fields=Signature,TestField'
        :return: dict
        """
        endpoint = FORM_TEMPLATES_URL + '/' + template_id + '/' + FORMS_URL + '/' + instance_id

        if query_string:
            endpoint += '?' + query_string

        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_form_instance_pdf_file_stream(self, template_id, instance_id):
        """
        requests a file stream of a form instance converted to a PDF
        :param template_id: string uuid4
        :param instance_id: string uuid4
        :return: stream
        """
        endpoint = FORM_TEMPLATES_URL + '/' + template_id + '/' + FORMS_URL + '/' + instance_id + '/' + PDF_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        stream = requests.get(request_url, headers=headers, stream=True)

        return stream

    def get_form_template_fields(self, template_id):
        """
        get form fields for a form template
        :param template_id: string uuid4
        :return: dict
        """
        endpoint = FORM_TEMPLATES_URL + '/' + template_id + '/' + FIELDS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_form_instances(self, template_id, query='', query_string=''):
        """
        get all form instances for a form template,
        search for particular instance, request form field values,
        or all of the above
        :param template_id: string uuid4
        :param query: string, example: instanceName='testForm-000026'
        :param query_string: string, example: 'fields=DocumentType'
        :return: dict
        """
        endpoint = FORM_TEMPLATES_URL + '/' + template_id + '/' + FORMS_URL

        if query and query_string:
            endpoint += '?q=' + query + '&' + query_string
        elif query and not query_string:
            endpoint += '?q=' + query
        elif query_string and not query:
            endpoint += '?' + query_string

        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def create_form_instance(self, template_id, fields_dict):
        """
        create a form instance of a form template
        by filling out a form with the provided fields_dict
        :param template_id: string uuid4
        :param fields_dict: dict, example: {'field1': '5', 'field2': True, 'field3': 5}
        :return: dict
        """
        endpoint = FORM_TEMPLATES_URL + '/' + template_id + '/' + FORMS_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.post(request_url, headers=headers, data=fields_dict).json()

        return resp

    def create_form_instance_revision(self, template_id, instance_id, fields_dict):
        """
        create a form instance revision of a form template
        by filling out a form with the provided fields_dict
        :param template_id: string uuid4
        :param instance_id: sting uuid4
        :param fields_dict: dict, example: {'field1': '5', 'field2': True, 'field3': 5}
        :return: dict
        """
        endpoint = FORM_TEMPLATES_URL + '/' + template_id + '/' + FORMS_URL + '/' + instance_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.post(request_url, headers=headers, data=fields_dict).json()

        return resp

    def relate_form(self, instance_id1, instance_id2):
        """
        relate a form instance to another form instance
        :param instance_id1: string uuid4
        :param instance_id2: string uuid4
        :return: dict
        """
        endpoint = FORM_INSTANCE_URL + '/' + instance_id1 + '/' + RELATE_FORM_URL + instance_id2
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers).json()

        return resp

    def relate_document(self, instance_id, document_id):
        """
        relate a form instance to a document
        :param instance_id: string uuid4
        :param document_id: string uuid4
        :return: dict
        """
        endpoint = FORM_INSTANCE_URL + '/' + instance_id + '/' + RELATE_DOCUMENT_URL + document_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers).json()

        return resp

    def relate_project(self, instance_id, project_id):
        """
        relate a form instance to a project
        :param instance_id: string uuid4
        :param project_id: string uuid4
        :return: dict
        """
        endpoint = FORM_INSTANCE_URL + '/' + instance_id + '/' + RELATE_PROJECT_URL + project_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers).json()

        return resp

    def unrelate_form(self, instance_id1, instance_id2):
        """
        unrelate a form instance to another form instance
        :param instance_id1: string uuid4
        :param instance_id2: string uuid4
        :return: dict
        """
        endpoint = FORM_INSTANCE_URL + '/' + instance_id1 + '/' + UNRELATE_FORM_URL + instance_id2
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers).json()

        return resp

    def unrelate_document(self, instance_id, document_id):
        """
        unrelate a form instance to a document
        :param instance_id: string uuid4
        :param document_id: string uuid4
        :return: dict
        """
        endpoint = FORM_INSTANCE_URL + '/' + instance_id + '/' + UNRELATE_DOCUMENT_URL + document_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers).json()

        return resp

    def unrelate_project(self, instance_id, project_id):
        """
        unrelate a form instance to a project
        :param instance_id: string uuid4
        :param project_id: string uuid4
        :return: dict
        """
        endpoint = FORM_INSTANCE_URL + '/' + instance_id + '/' + UNRELATE_PROJECT_URL + project_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers).json()

        return resp

    def get_embedded_form(self, web_token, instance_id):
        """
        returns a url that can be embedded in an i-frame that will launch a form instance
        :param web_token: string uuid4, example: web_token returned from UserService.get_user_web_token
        :param instance_id: string uuid4
        :return: string
        """
        login_url = self.vault.url + '/' + VV_LOGIN_URL + web_token  # authentication portion of form url
        return_url = RETURN_URL + instance_id + HIDE_MENU_URL  # form instance portion of form url
        form_url = login_url + return_url

        return form_url

    def get_related_documents(self, instance_id, query_string=''):
        """
        get related documents of a form instance
        :param instance_id: string uuid4
        :param query_string: string, example: 'indexFields=include'
        :return: dict
        """
        endpoint = FORM_INSTANCE_URL + '/' + instance_id + '/' + DOCUMENTS_URL

        if query_string:
            endpoint += '?' + query_string

        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_related_forms(self, instance_id, query_string=''):
        """
        get related forms of a form instance
        :param instance_id: string uuid4
        :param query_string: string, example: "q=[instanceName]='TEST-00000064'"
        :return: dict
        """
        endpoint = FORM_INSTANCE_URL + '/' + instance_id + '/' + FORMS_URL

        if query_string:
            endpoint += '?' + query_string

        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp



import requests
from ..constants import GROUPS_URL, USERS_URL


class GroupService:
	def __init__(self, vault):
		self.vault = vault

	def get_groups(self, query=''):
		"""
		get all groups or search by query
		:param query: string, example: TODO: query example
		:return: dict
		"""
		request_url = self.vault.base_url + GROUPS_URL
		if query:
			request_url += '?q=' + query

		headers = self.vault.get_auth_headers()
		resp = requests.get(request_url, headers=headers).json()

		return resp

	def get_group(self, group_id):
		"""
		get a group by id
		:param group_id: string uuid4
		:return: dict
		"""
		endpoint = GROUPS_URL + '/' + group_id
		request_url = self.vault.base_url + endpoint
		headers = self.vault.get_auth_headers()
		resp = requests.get(request_url, headers=headers).json()

		return resp

	def get_group_users(self, group_id):
		"""
		get users of a group by id
		:param group_id: string uuid4
		:return: dict
		"""
		endpoint = GROUPS_URL + '/' + group_id + '/' + USERS_URL
		request_url = self.vault.base_url + endpoint
		headers = self.vault.get_auth_headers()
		resp = requests.get(request_url, headers=headers).json()

		return resp

	def get_group_user(self, group_id, user_id):
		"""
		get a user of a group by ids
		:param group_id: string uuid4
		:param user_id: string uuid4
		:return: dict
		"""
		endpoint = GROUPS_URL + '/' + group_id + '/' + USERS_URL + '/' + user_id
		request_url = self.vault.base_url + endpoint
		headers = self.vault.get_auth_headers()
		resp = requests.get(request_url, headers=headers).json()

		return resp

	def create_group(self, name, description, site_id):
		"""
		create a new group
		:param name: string
		:param description: string
		:param site_id: string uuid4
		:return: dict
		"""
		request_url = self.vault.base_url + GROUPS_URL
		headers = self.vault.get_auth_headers()

		payload = {
			'name': name,
			'description': description,
			'siteId': site_id
		}

		resp = requests.post(request_url, data=payload, headers=headers).json()

		return resp

	def update_group(self, group_id, name, description):
		"""
		create a new group
		:param group_id: string uuid4
		:param name: string
		:param description: string
		:return: dict
		"""
		endpoint = GROUPS_URL + '/' + group_id
		request_url = self.vault.base_url + endpoint
		headers = self.vault.get_auth_headers()

		payload = {
			'name': name,
			'description': description
		}

		resp = requests.put(request_url, data=payload, headers=headers).json()

		return resp

	def add_user_to_group(self, group_id, user_id):
		"""
		adds a user to a group
		:param group_id: string uuid4
		:param user_id: string uuid4
		:return: dict
		"""
		endpoint = GROUPS_URL + '/' + group_id + '/' + USERS_URL + '/' + user_id
		request_url = self.vault.base_url + endpoint
		headers = self.vault.get_auth_headers()
		resp = requests.put(request_url, headers=headers).json()

		return resp



from ..constants import TOKEN_URL, GRANT_TYPE_PASSWORD, GRANT_TYPE_REFRESH
import requests


class AuthService:
    def __init__(self, url, customer_alias, database_alias, client_id, client_secret, user_web_token):
        """
        :param url: string, example: https://demo.visualvault.com
        :param customer_alias: string
        :param database_alias: string
        :param client_id: string UUID(version=4)
        :param client_secret: string, example: khN18YAZPe6F3Z0tc2W0HXCb487jm0wgwe6kNffUNf0=
        :param user_web_token: string UUID(version=4), passed in if authentication is user impersonation
        """
        self.url = url
        self.customer_alias = customer_alias
        self.database_alias = database_alias
        self.client_id = client_id
        self.client_secret = client_secret
        self.user_web_token = user_web_token

    def get_access_token(self):
        """
        requests access_token
        :return: dict
        """
        request_url = self.url + TOKEN_URL
        headers = {'Content-Type': 'application/json'}

        payload = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': GRANT_TYPE_PASSWORD
        }

        if self.user_web_token:  # impersonation
            payload['username'] = self.user_web_token
            payload['password'] = self.user_web_token
        else:
            payload['username'] = self.client_id
            payload['password'] = self.client_secret

        resp = requests.post(request_url, data=payload, headers=headers).json()

        return resp

    def refresh_access_token(self, refresh_token):
        """
        requests refresh of access_token
        :param refresh_token: string
        :return: dict
        """
        request_url = self.url + TOKEN_URL
        headers = {'Content-Type': 'application/json'}

        payload = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'refresh_token': refresh_token,
            'grant_type': GRANT_TYPE_REFRESH
        }

        resp = requests.post(request_url, data=payload, headers=headers).json()

        return resp



import requests
from ..constants import USERS_URL, WEB_TOKEN_URL, SITE_ID_URL, JWT_URL


class UserService:
    def __init__(self, vault):
        self.vault = vault

    def get_users(self, query=''):
        """
        get all users or search by query
        :param query: string, default: empty string, example: "userId='test@test.com'"
        :return: dict
        """
        request_url = self.vault.base_url + USERS_URL
        if query:
            request_url += '?q=' + query

        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()
        return resp

    def get_user(self, user_id):
        """
        get a single user
        :param user_id: string uuid4
        :return: dict
        """
        endpoint = USERS_URL + '/' + user_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def get_user_web_token(self, user_id):
        """
        get a users webToken by userId
        :param user_id: string uuid4
        :return: dict
        """
        endpoint = USERS_URL + '/' + user_id + '/' + WEB_TOKEN_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def create_user(self, site_id, user_id, first_name, last_name, email, password, must_change_password=False,
                    send_email=False, password_never_expires=True):
        """
        create a new user
        :param site_id: string uuid4
        :param user_id: string uuid4
        :param first_name: string
        :param last_name: string
        :param email: string
        :param password: string
        :param must_change_password: bool, default False
        :param send_email: bool, default False
        :param password_never_expires: bool, default True
        :return: dict
        """
        endpoint = USERS_URL + SITE_ID_URL + site_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()

        payload = {
            'userId': user_id,
            'firstName': first_name,
            'lastName': last_name,
            'emailAddress': email,
            'password': password,
            'mustChangePassword': must_change_password,
            'sendEmail': send_email,
            'passwordNeverExpires': password_never_expires
        }

        resp = requests.post(request_url, headers=headers, data=payload).json()

        return resp

    def update_user(self, user_id, fields_dict):
        """
        update a user
        :param user_id: string uuid4
        :param fields_dict: dict, example: {'enabled': True, 'lastName': 'new name'}
        :return: dict
        """
        endpoint = USERS_URL + '/' + user_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers, data=fields_dict).json()

        return resp

    def get_user_jwt(self):
        """
        get a users JWT by userId
        refreshes a users JWT if JWT is passed into Vault object
        :return: dict
        """
        endpoint = USERS_URL + '/' + JWT_URL
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp



class Token:
    def __init__(self, access_token, token_expiration, refresh_token):
        """
        :param access_token: string
        :param token_expiration: datetime
        :param refresh_token: string
        """
        self.access_token = access_token
        self.token_expiration = token_expiration
        self.refresh_token = refresh_token



import requests
from ..constants import INDEX_FIELDS_URL, FOLDERS_URL


class IndexFieldService:
    def __init__(self, vault):
        self.vault = vault

    def get_index_fields(self, query=''):
        """
        get all index fields or filter by query
        :param query: string, example: "label = 'TestField'"
        :return: dict
        """
        request_url = self.vault.base_url + INDEX_FIELDS_URL
        if query:
            request_url += '?q=' + query

        headers = self.vault.get_auth_headers()
        resp = requests.get(request_url, headers=headers).json()

        return resp

    def create_index_field(self, label, description, field_type, required, default_value, query_value_field='',
                           query_display_field='', query_id='00000000-0000-0000-0000-000000000000',
                           drop_down_list_id='00000000-0000-0000-0000-000000000000'):
        """
        creates a new index field definition
        :param label: string
        :param description: string
        :param field_type: int example: TODO: document field_type ENUM
        :param required: bool
        :param default_value: string
        :param query_value_field: string, default: empty string
        :param query_display_field: string, default: empty string
        :param query_id: string uuid4, default: empty uuid4
        :param drop_down_list_id: string uuid4, default: empty uuid4
        :return: dict
        """
        request_url = self.vault.base_url + INDEX_FIELDS_URL
        headers = self.vault.get_auth_headers()

        payload = {
            'label': label,
            'description': description,
            'fieldType': field_type,
            'queryId': query_id,
            'dropDownListId': drop_down_list_id,
            'queryValueField': query_value_field,
            'queryDisplayField': query_display_field,
            'required': required,
            'defaultValue': default_value
        }

        resp = requests.post(request_url, headers=headers, data=payload).json()

        return resp

    def update_index_field(self, index_field_id, label, description, required, default_value, query_value_field='',
                           query_display_field='', query_id='00000000-0000-0000-0000-000000000000',
                           drop_down_list_id='00000000-0000-0000-0000-000000000000'):
        """
        updates an index field definition
        :param index_field_id: string uuid4
        :param label: string
        :param description: string
        :param required: bool
        :param default_value: string
        :param query_value_field: string, default: empty string
        :param query_display_field: string, default: empty string
        :param query_id: string uuid4, default: empty uuid4
        :param drop_down_list_id: string uuid4, default: empty uuid4
        :return: dict
        """
        request_url = self.vault.base_url + INDEX_FIELDS_URL + '/' + index_field_id
        headers = self.vault.get_auth_headers()

        payload = {
            'label': label,
            'description': description,
            'queryId': query_id,
            'dropDownListId': drop_down_list_id,
            'queryValueField': query_value_field,
            'queryDisplayField': query_display_field,
            'required': required,
            'defaultValue': default_value
        }

        resp = requests.put(request_url, headers=headers, data=payload).json()

        return resp

    def relate_index_field_to_folder(self, field_id, folder_id):
        endpoint = INDEX_FIELDS_URL + '/' + field_id + '/' + FOLDERS_URL + '/' + folder_id
        request_url = self.vault.base_url + endpoint
        headers = self.vault.get_auth_headers()
        resp = requests.put(request_url, headers=headers).json()

        return resp



import requests
from ..constants import EMAILS_URL


class EmailService:
    def __init__(self, vault):
        """
        :param vault: Vault
        """
        self.vault = vault

    def send_email(self, recipient, cc_recipient, subject, body, has_attachments, documents):
        """
        :param recipient: string, example: test@test.com
        :param cc_recipient: string, example: test@test.com,test2@aol.com
        :param subject: string
        :param body: string
        :param has_attachments: bool
        :param documents: list(string uuid4)
        :return: dict
        """
        request_url = self.vault.base_url + EMAILS_URL
        headers = self.vault.get_auth_headers()

        payload = {
            'recipients': recipient,
            'ccrecipients': cc_recipient,
            'subject': subject,
            'body': body,
            'hasAttachments': has_attachments,
            'documents': documents
        }

        resp = requests.post(request_url, data=payload, headers=headers).json()

        return resp



from .token import Token
from .utilities import get_token_expiration
from .services.auth_service import AuthService


class Vault:
    def __init__(self, url, customer_alias, database_alias, client_id, client_secret, user_web_token=None, jwt=None):
        """
        if user_web_token is passed in, then vv will authenticate on behalf of the user that
        the web_token belongs to. if user_web_token is not passed in (default=None), then
        vv will authenticate on behalf of the user that the client_id and client_secret belong to.
        :param url: string, example: https://demo.visualvault.com
        :param customer_alias: str
        :param database_alias: str
        :param client_id: str, UUID(version=4)
        :param client_secret: str, example: khN18YAZPe6F3Z0tc2W0HXCb487jm0wgwe6kNffUNf0=
        :param user_web_token: str UUID(version=4), passed in if authentication is user impersonation
        :param jwt: string, JSON Web Token
        """
        self.url = url
        self.customer_alias = customer_alias
        self.database_alias = database_alias
        self.client_id = client_id
        self.client_secret = client_secret
        self.user_web_token = user_web_token
        self.jwt = jwt
        self.token = self.get_access_token()
        self.base_url = self.get_base_url()

    def get_access_token(self):
        """
        requests access token
        :return: Token
        """
        authentication_service = AuthService(self.url, self.customer_alias, self.database_alias, self.client_id,
                                             self.client_secret, self.user_web_token)

        if not self.jwt:
            resp = authentication_service.get_access_token()
            access_token = resp['access_token']
            token_expiration = get_token_expiration(resp['expires_in'])
            refresh_token = resp['refresh_token']
        else:
            access_token = self.jwt
            token_expiration = None
            refresh_token = None

        token = Token(access_token, token_expiration, refresh_token)

        return token

    def get_base_url(self):
        """
        :return: string
        """
        base_url = self.url + '/api/v1/' + self.customer_alias + '/' + self.database_alias + '/'

        return base_url

    def refresh_access_token(self):
        """
        void method that refreshes Vault.token
        :return: None
        """
        authentication_service = AuthService(self.url, self.customer_alias, self.database_alias, self.client_id,
                                             self.client_secret, self.user_web_token)

        resp = authentication_service.refresh_access_token(self.token.refresh_token)
        access_token = resp['access_token']
        token_expiration = get_token_expiration(resp['expires_in'])
        refresh_token = resp['refresh_token']
        self.token = Token(access_token, token_expiration, refresh_token)

    def get_auth_headers(self):
        """
        :return: dict
        """
        headers = {'Authorization': 'Bearer ' + self.token.access_token}

        return headers
